@@ -46,7 +46,8 @@ class AlphaDropout(Layer):
         if rate > 0:
             self.seed_generator = backend.random.SeedGenerator(seed)
         self.supports_masking = True
-        self.built = True
+
+        self._build_at_init()
 
     def call(self, inputs, training=False):
         if training and self.rate > 0:
