@@ -6,7 +6,6 @@ import numpy as np
 from absl import logging
 
 from keras.src import backend
-from keras.src import optimizers
 from keras.src.backend.common import global_state
 from keras.src.legacy.saving import json_utils
 from keras.src.legacy.saving import saving_options
@@ -161,6 +160,8 @@ def load_model_from_hdf5(filepath, custom_objects=None, compile=True):
             # Set optimizer weights.
             if "optimizer_weights" in f:
                 try:
+                    from keras.src import optimizers
+
                     if isinstance(model.optimizer, optimizers.Optimizer):
                         model.optimizer.build(model._trainable_variables)
                     else:
@@ -249,6 +250,8 @@ def save_optimizer_weights_to_hdf5_group(hdf5_group, optimizer):
         hdf5_group: HDF5 group.
         optimizer: optimizer instance.
     """
+    from keras.src import optimizers
+
     if isinstance(optimizer, optimizers.Optimizer):
         symbolic_weights = optimizer.variables
     else:
@@ -315,12 +318,14 @@ def save_attributes_to_hdf5_group(group, name, data):
         group.attrs[name] = data
 
 
-def load_weights_from_hdf5_group(f, model):
+def load_weights_from_hdf5_group(f, model, skip_mismatch=False):
     """Implements topological (order-based) weight loading.
 
     Args:
         f: A pointer to a HDF5 group.
         model: Model instance.
+        skip_mismatch: Boolean, whether to skip loading of weights
+            where there is a mismatch in the shape of the weights,
 
     Raises:
         ValueError: in case of mismatch between provided layers
@@ -376,6 +381,7 @@ def load_weights_from_hdf5_group(f, model):
             layer,
             symbolic_weights,
             weight_values,
+            skip_mismatch=skip_mismatch,
             name=f"layer #{k} (named {layer.name})",
         )
 
@@ -400,6 +406,7 @@ def load_weights_from_hdf5_group(f, model):
             model,
             symbolic_weights,
             weight_values,
+            skip_mismatch=skip_mismatch,
             name="top-level model",
         )
 
