@@ -2,10 +2,13 @@ import importlib
 
 
 class LazyModule:
-    def __init__(self, name, pip_name=None):
+    def __init__(self, name, pip_name=None, import_error_msg=None):
         self.name = name
-        pip_name = pip_name or name
-        self.pip_name = pip_name
+        self.pip_name = pip_name or name
+        self.import_error_msg = import_error_msg or (
+            f"This requires the {self.name} module. "
+            f"You can install it via `pip install {self.pip_name}`"
+        )
         self.module = None
         self._available = None
 
@@ -23,10 +26,7 @@ class LazyModule:
         try:
             self.module = importlib.import_module(self.name)
         except ImportError:
-            raise ImportError(
-                f"This requires the {self.name} module. "
-                f"You can install it via `pip install {self.pip_name}`"
-            )
+            raise ImportError(self.import_error_msg)
 
     def __getattr__(self, name):
         if name == "_api_export_path":
@@ -44,6 +44,17 @@ gfile = LazyModule("tensorflow.io.gfile", pip_name="tensorflow")
 tensorflow_io = LazyModule("tensorflow_io")
 scipy = LazyModule("scipy")
 jax = LazyModule("jax")
-torchvision = LazyModule("torchvision")
+torch_xla = LazyModule(
+    "torch_xla",
+    import_error_msg=(
+        "This requires the torch_xla module. You can install it via "
+        "`pip install torch-xla`. Additionally, you may need to update "
+        "LD_LIBRARY_PATH if necessary. Torch XLA builds a shared library, "
+        "_XLAC.so, which needs to link to the version of Python it was built "
+        "with. Use the following command to update LD_LIBRARY_PATH: "
+        "`export LD_LIBRARY_PATH=<path to Python>/lib:$LD_LIBRARY_PATH`"
+    ),
+)
 optree = LazyModule("optree")
 dmtree = LazyModule("tree")
+tf2onnx = LazyModule("tf2onnx")
